@@ -18,18 +18,18 @@
 #include <lua.h>
 #include <lualib.h>
 
-#include <common/time.h>
-#include <common/uri_auth.h>
-
-#include <types/cli.h>
-#include <types/hlua.h>
-#include <types/proxy.h>
-#include <types/stats.h>
-
-#include <proto/proxy.h>
-#include <proto/server.h>
-#include <proto/stats.h>
-#include <proto/stick_table.h>
+#include <haproxy/cli-t.h>
+#include <haproxy/errors.h>
+#include <haproxy/hlua-t.h>
+#include <haproxy/http.h>
+#include <haproxy/net_helper.h>
+#include <haproxy/pattern-t.h>
+#include <haproxy/proxy.h>
+#include <haproxy/regex.h>
+#include <haproxy/server.h>
+#include <haproxy/stats.h>
+#include <haproxy/stick_table.h>
+#include <haproxy/time.h>
 
 /* Contains the class reference of the concat object. */
 static int class_concat_ref;
@@ -39,7 +39,6 @@ static int class_listener_ref;
 static int class_regex_ref;
 static int class_stktable_ref;
 
-#define MAX_STK_FILTER_LEN 4
 #define STATS_LEN (MAX((int)ST_F_TOTAL_FIELDS, (int)INF_TOTAL_FIELDS))
 
 static THREAD_LOCAL struct field stats[STATS_LEN];
@@ -123,7 +122,7 @@ int hlua_fcn_pushfield(lua_State *L, struct field *field)
 		/* 64 bits case, S64 is always supported */
 		lua_pushinteger(L, field->u.s64);
 #else
-		/* 64 bits case, S64 is supported beetween INT_MIN and INT_MAX */
+		/* 64 bits case, S64 is supported between INT_MIN and INT_MAX */
 		if (field->u.s64 < INT_MIN || field->u.s64 > INT_MAX)
 			lua_pushnumber(L, (lua_Number)field->u.s64);
 		else
@@ -214,7 +213,7 @@ int hlua_dump_object(struct lua_State *L)
 
 /* This function register a table as metatable and. It names
  * the metatable, and returns the associated reference.
- * The original table is poped from the top of the stack.
+ * The original table is popped from the top of the stack.
  * "name" is the referenced class name.
  */
 int hlua_register_metatable(struct lua_State *L, char *name)
@@ -235,7 +234,7 @@ int hlua_register_metatable(struct lua_State *L, char *name)
 	lua_rawset(L, -3);
 
 	/* Register a named entry for the table. The table
-	 * reference is copyed first because the function
+	 * reference is copied first because the function
 	 * lua_setfield() pop the entry.
 	 */
 	lua_pushvalue(L, -1);
@@ -424,7 +423,7 @@ static int hlua_concat_dump(lua_State *L)
 	buffer = lua_touserdata(L, -1);
 	lua_pop(L, 1);
 
-	/* Push the soncatenated strng in the stack. */
+	/* Push the soncatenated string in the stack. */
 	lua_pushlstring(L, buffer, b->len);
 	return 1;
 }
@@ -588,6 +587,12 @@ static void hlua_stktable_entry(lua_State *L, struct stktable *t, struct stksess
 			lua_pushinteger(L, read_freq_ctr_period(&stktable_data_cast(ptr, std_t_frqp),
 			                t->data_arg[dt].u));
 			break;
+		case STD_T_DICT: {
+			struct dict_entry *de;
+			de = stktable_data_cast(ptr, std_t_dict);
+			lua_pushstring(L, de ? (char *)de->value.key : "-");
+			break;
+		}
 		}
 
 		lua_settable(L, -3);
@@ -611,7 +616,7 @@ int hlua_stktable_lookup(lua_State *L)
 	t = hlua_check_stktable(L, 1);
 	smp.data.type = SMP_T_STR;
 	smp.flags = SMP_F_CONST;
-	smp.data.u.str.area = (char *)luaL_checkstring(L, 2);
+	smp.data.u.str.area = (char *)lua_tolstring(L, 2, &smp.data.u.str.data);
 
 	skey = smp_to_stkey(&smp, t);
 	if (!skey) {
@@ -682,7 +687,7 @@ int hlua_stktable_dump(lua_State *L)
 	int op;
 	int dt;
 	long long val;
-	struct stk_filter filter[MAX_STK_FILTER_LEN];
+	struct stk_filter filter[STKTABLE_FILTER_LEN];
 	int filter_count = 0;
 	int i;
 	int skip_entry;
@@ -700,8 +705,8 @@ int hlua_stktable_dump(lua_State *L)
 		while (lua_next(L, 2) != 0) {
 			int entry_idx = 0;
 
-			if (filter_count >= MAX_STK_FILTER_LEN)
-				return hlua_error(L, "Filter table too large (len > %d)", MAX_STK_FILTER_LEN);
+			if (filter_count >= STKTABLE_FILTER_LEN)
+				return hlua_error(L, "Filter table too large (len > %d)", STKTABLE_FILTER_LEN);
 
 			if (lua_type(L, -1) != LUA_TTABLE  || lua_rawlen(L, -1) != 3)
 				return hlua_error(L, "Filter table entry must be a triplet: {\"data_col\", \"op\", val} (entry #%d)", filter_count + 1);
@@ -1034,13 +1039,18 @@ int hlua_server_set_addr(lua_State *L)
 {
 	struct server *srv;
 	const char *addr;
+	const char *port;
 	const char *err;
 
 	srv = hlua_check_server(L, 1);
 	addr = luaL_checkstring(L, 2);
+	if (lua_gettop(L) >= 3)
+		port = luaL_checkstring(L, 3);
+	else
+		port = NULL;
 
 	HA_SPIN_LOCK(SERVER_LOCK, &srv->lock);
-	err = server_parse_addr_change_request(srv, addr, "Lua script");
+	err = update_server_addr_port(srv, addr, port, "Lua script");
 	HA_SPIN_UNLOCK(SERVER_LOCK, &srv->lock);
 	if (!err)
 		lua_pushnil(L);
@@ -1421,7 +1431,7 @@ int hlua_fcn_post_init(lua_State *L)
  * It tokenize the input string using the list of separators
  * as separator.
  *
- * The functionreturns a tablle filled with tokens.
+ * The functionreturns a table filled with tokens.
  */
 int hlua_tokenize(lua_State *L)
 {
@@ -1521,10 +1531,10 @@ int hlua_match_addr(lua_State *L)
 		int i;
 
 		for (i = 0; i < 16; i += 4) {
-			if ((*(uint32_t *)&addr1->addr.v6.ip.s6_addr[i] &
-			     *(uint32_t *)&addr2->addr.v6.mask.s6_addr[i]) !=
-			    (*(uint32_t *)&addr2->addr.v6.ip.s6_addr[i] &
-			     *(uint32_t *)&addr1->addr.v6.mask.s6_addr[i]))
+			if ((read_u32(&addr1->addr.v6.ip.s6_addr[i]) &
+			     read_u32(&addr2->addr.v6.mask.s6_addr[i])) !=
+			    (read_u32(&addr2->addr.v6.ip.s6_addr[i]) &
+			     read_u32(&addr1->addr.v6.mask.s6_addr[i])))
 				break;
 		}
 		if (i == 16) {
