@@ -98,6 +98,11 @@ int sample_load_map(struct arg *arg, struct sample_conv *conv,
 {
 	struct map_descriptor *desc;
 
+	if (!(global.mode & MODE_STARTING)) {
+		memprintf(err, "map: cannot load map at runtime");
+		return 0;
+	}
+
 	/* create new map descriptor */
 	desc = map_create_descriptor(conv);
 	if (!desc) {
@@ -146,6 +151,7 @@ int sample_load_map(struct arg *arg, struct sample_conv *conv,
 				  arg[1].data.str.area);
 			return 0;
 		}
+		chunk_destroy(&arg[1].data.str);
 		if (data.type == SMP_T_IPV4) {
 			arg[1].type = ARGT_IPV4;
 			arg[1].data.ipv4 = data.u.ipv4;
@@ -156,6 +162,7 @@ int sample_load_map(struct arg *arg, struct sample_conv *conv,
 	}
 
 	/* replace the first argument by this definition */
+	chunk_destroy(&arg[0].data.str);
 	arg[0].type = ARGT_MAP;
 	arg[0].data.map = desc;
 
